function varargout = epc(varargin)
% EPC MATLAB code for epc.fig
%      EPC, by itself, creates a new EPC or raises the existing
%      singleton*.
%
%      H = EPC returns the handle to a new EPC or the handle to
%      the existing singleton*.
%
%      EPC('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in EPC.M with the given input arguments.
%
%      EPC('Property','Value',...) creates a new EPC or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before epc_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to epc_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help epc

% Last Modified by GUIDE v2.5 06-Sep-2013 12:15:27

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @epc_OpeningFcn, ...
                   'gui_OutputFcn',  @epc_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before epc is made visible.
function epc_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to epc (see VARARGIN)

% Choose default command line output for epc
handles.output = hObject;

% Define constants and initial values
handles.earth_radius = 6.378e6; % The Earth radius in meter
handles.t_test_sig_level = 0.05; % Student's t-test significance level

handles.input_tbl_data = false;
handles.output_tbl_data = false;

% Define external applications
handles.unix_pdf_reader = 'acroread';

% Define error messages
handles.error_msg_0 = 'The input data has not yet been loaded.';
handles.error_msg_1 = 'The output data has not yet been calculated.';
handles.error_msg_2 = 'Illegal character(s) or empty field(s) in the Euler Pole parameters.';
handles.error_msg_3 = 'The square of the a-posteriori sigma value is negative. Results might be incorrect.';
handles.error_msg_4 = 'The help file can not be found.';
handles.error_msg_5 = 'The input Euler parameters are not in the proper range.';
handles.error_msg_6 = 'This test is only available for estimating the Euler pole parameters.';

% Add the software directories to the MATLAB search path
handles.installPath = fileparts(mfilename('fullpath'));

handles.dlgDir = fullfile(handles.installPath, 'dlg');
handles.funcDir = fullfile(handles.installPath, 'func');
handles.helpDir = fullfile(handles.installPath, 'help');
handles.dataDir = fullfile(handles.installPath, 'data');

addpath(handles.installPath, handles.dlgDir, handles.funcDir);

% Capture mouse movement in the current axis
set(hObject, 'WindowButtonMotionFcn', {@capture_mouse_movement, handles});

% Remove empty lines from input and output tables
set([handles.input_tbl, handles.output_tbl], 'Data', []);

% Plot the world map and the tectonic plates
handles.continents = load(fullfile(handles.dataDir, 'continent.mat'));
handles.tectonic = load(fullfile(handles.dataDir, 'tec_plates.mat'));

hold(handles.map_canvas, 'on');
handles.world_map = plot(handles.map_canvas, ...
    handles.continents.X, handles.continents.Y, ...
    'Color', [0.1 0.7 0.1]);
handles.tectonic_map = plot(handles.map_canvas, ...
    handles.tectonic.X, handles.tectonic.Y, ...
    'Color', [0.6 0.6 0.6]);
hold(handles.map_canvas, 'off');

set(handles.map_canvas, ...
    'xlim', [min(handles.continents.X), max(handles.continents.X)], ...
    'ylim', [min(handles.continents.Y), max(handles.continents.Y)], ...
    'xGrid', 'On', ...
    'yGrid', 'On', ...
    'xAxisLocation', 'Top', ...
    'yAxisLocation', 'Right');

% Update handles structure
guidata(hObject, handles);

% Make the map canvas current
axes(handles.map_canvas);

% Disable the reload pushtool
set(handles.reload_pushtool, 'Enable', 'Off');

% UIWAIT makes epc wait for user response (see UIRESUME)
% uiwait(handles.epc);


% --- Outputs from this function are returned to the command line.
function varargout = epc_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --------------------------------------------------------------------
function file_menu_Callback(hObject, eventdata, handles)
% hObject    handle to file_menu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function test_menu_Callback(hObject, eventdata, handles)
% hObject    handle to test_menu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function help_menu_Callback(hObject, eventdata, handles)
% hObject    handle to help_menu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function users_manual_item_Callback(hObject, eventdata, handles)
% hObject    handle to users_manual_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

handles.help_file = fullfile(handles.helpDir, 'users_manual.pdf');
if ~exist(handles.help_file, 'file')
    errordlg(handles.error_msg_4, 'Error ...', 'Modal');
    return;
end

% Opens the help file with the external pdf reader
if isunix
    [status, result] = system([handles.unix_pdf_reader, ' ', handles.help_file]);
    if status
        errordlg(result, 'Error ...', 'modal');
        return;
    end
else
    try
        open(handles.help_file);
    catch error_msg
        errordlg(error_msg.message, 'Error ...', 'Modal');
        return;
    end
end

% Update handles structure
guidata(hObject, handles);



% --------------------------------------------------------------------
function about_item_Callback(hObject, eventdata, handles)
% hObject    handle to about_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

about_epc_dlg;

return;

% --------------------------------------------------------------------
function open_item_Callback(hObject, eventdata, handles)
% hObject    handle to open_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[handles.file_name, handles.path_name, handles.filter_index] = uigetfile(...
    {'*.ecf', 'ECEF Files (*.ecf)'; '*.neu', 'Topocentric Files (*.neu)'}, 'Select a file ...');

if handles.filter_index
    cd(handles.path_name);
end

handles = write_input_tbl(hObject, eventdata, handles);

% Update handles structure
guidata(hObject, handles);



% --------------------------------------------------------------------
function exit_item_Callback(hObject, eventdata, handles)
% hObject    handle to exit_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

selection = ...
   questdlg('Do you really want to quit?',...
            'Close EPC ...', ...
            'Yes','No','No');
if ~strcmpi(selection, 'Yes')
    return;
end

delete(handles.epc);


% --------------------------------------------------------------------
function open_pushtool_ClickedCallback(hObject, eventdata, handles)
% hObject    handle to open_pushtool (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

open_item_Callback(hObject, eventdata, handles);

return;


% --------------------------------------------------------------------
function view_menu_Callback(hObject, eventdata, handles)
% hObject    handle to view_menu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function view_stations_item_Callback(hObject, eventdata, handles)
% hObject    handle to view_stations_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the input table is already loaded
if isequal(handles.input_tbl_data, false)
    errordlg(handles.error_msg_0, 'Error ...',  'Modal');
    return;
elseif strcmpi(get(handles.view_stations_item, 'Checked'), 'On')
    set(handles.view_stations_item, 'Checked', 'off');
    delete(handles.stn_symboles);
    return;
end

switch handles.filter_index
    case 1
        stn_coordinates = xyz2ell(handles.positions(:, 1), handles.positions(:, 2), handles.positions(:, 3));
        
    case 2
        stn_coordinates = handles.positions;

    otherwise
            return;
end

% Plot the stations
hold(handles.map_canvas, 'on');
handles.stn_symboles = plot(handles.map_canvas, stn_coordinates(:, 2), stn_coordinates(:, 1), '^',...
    'MarkerEdgeColor','k',...
    'MarkerFaceColor','r');
hold(handles.map_canvas, 'off');

% Set the menu item checked
set(handles.view_stations_item, 'Checked', 'On');

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function calculate_menu_Callback(hObject, eventdata, handles)
% hObject    handle to calculate_menu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function calculate_velocities_item_Callback(hObject, eventdata, handles)
% hObject    handle to calculate_velocities_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the input data was already loaded
if isequal(handles.input_tbl_data, false)
    errordlg(handles.error_msg_0, 'Error ...',  'Modal');
    return;
end

% Get the Euler parameters
handles.euler_latitude = str2double(get(handles.euler_pole_latitude, 'String'));
handles.euler_longitude = str2double(get(handles.euler_pole_longitude, 'String'));
handles.euler_omega = str2double(get(handles.euler_pole_omega, 'String'));

handles.euler_latitude_sigma = str2double(get(handles.euler_pole_latitude_sigma, 'String'));
handles.euler_longitude_sigma = str2double(get(handles.euler_pole_longitude_sigma, 'String'));
handles.euler_omega_sigma = str2double(get(handles.euler_pole_omega_sigma, 'String'));

% Check the Euler parameters are correct
if any(isnan([handles.euler_latitude, handles.euler_longitude, handles.euler_omega, handles.euler_latitude_sigma, handles.euler_longitude_sigma, handles.euler_omega_sigma]))
    errordlg(handles.error_msg_2, 'Error ...',  'Modal');
    return;
elseif any([~isbetween(handles.euler_latitude, -90, 90); ~isbetween(handles.euler_longitude, -180, 180); ~gt([handles.euler_omega; handles.euler_latitude_sigma; handles.euler_longitude_sigma; handles.euler_omega_sigma], 0)])
    errordlg(handles.error_msg_5, 'Error ...',  'Modal');
    return;
end

% Calculate omega vector
handles.euler_parameters = [deg2rad(handles.euler_latitude), deg2rad(handles.euler_longitude), deg2rad(handles.euler_omega / 10^6)];
handles.euler_vector = euler_para2vect(handles.euler_parameters);

% Calculate the covariance matrix for the omega vector
handles.euler_parameters_cc = diag([deg2rad(handles.euler_latitude_sigma), deg2rad(handles.euler_longitude_sigma), deg2rad(handles.euler_omega_sigma / 10^6) ].^2);
handles.euler_dm = euler_para2dm(handles.euler_parameters);
handles.euler_vv = handles.euler_dm * handles.euler_parameters_cc * handles.euler_dm';

switch handles.filter_index
    case 1
        % Form the design matrix
        handles.dm = ecef2dm(handles.positions);
        
        % Calculate velocities in ECEF
        velocities = handles.dm * handles.euler_vector';
        velocities = [velocities(1:3:end), velocities(2:3:end), velocities(3:3:end)];
        d_velocities = handles.velocities - velocities;
        
        % Calculate coordinates of the stations
        lla = xyz2ell(handles.positions(:, 1), handles.positions(:, 2), handles.positions(:, 3));
        
        % Convert velocities in the topocentric system
        velocities = geocentric2topocentric(velocities, deg2rad(lla(:, 1:2)));
        d_velocities = geocentric2topocentric(d_velocities, deg2rad(lla(:, 1:2)));
        
        % Calculate the variance-covariance matrix for velocities in the ECEF system
        C_vv = handles.dm * handles.euler_vv * handles.dm';
        qq(length(C_vv), 3) = 0;
        for idx = 1:3:length(C_vv)
            qq(idx, 1)     = C_vv(idx, idx);
            qq(idx, 2)     = C_vv(idx, idx + 1);
            qq(idx, 3)     = C_vv(idx, idx + 2);
            
            qq(idx + 1, 1) = C_vv(idx + 1, idx);
            qq(idx + 1, 2) = C_vv(idx + 1, idx + 1);
            qq(idx + 1, 3) = C_vv(idx + 1, idx + 2);
            
            qq(idx + 2, 1) = C_vv(idx + 2, idx);
            qq(idx + 2, 2) = C_vv(idx + 2, idx + 1);
            qq(idx + 2, 3) = C_vv(idx + 2, idx + 2);
        end
        
        % Convert velocity variances to the topocentric system
        s_velocities = geocentric2topocentric_covmat(qq, deg2rad(lla(:, 1:2)));
        
        handles.output_positions  = lla(:, 1:2);
        handles.output_velocities = velocities(:, 1:2);
        handles.output_residuals  = d_velocities(:, 1:2);
        handles.output_sigma      = [sqrt(s_velocities(1:3:end, 1)), sqrt(s_velocities(2:3:end, 2)), s_velocities(1:3:end, 2)];
        
    case 2
        % Form the design matrix
        handles.dm = handles.earth_radius * topo2dm(deg2rad(handles.positions));
        
        % Calculate velocities in topocentric system
        velocities = handles.dm * handles.euler_vector';
        velocities = [velocities(1:2:end), velocities(2:2:end)];
        d_velocities = handles.velocities - velocities;
        
        % Calculate the variance-covariance matrix for velocities
        C_vv = handles.dm * handles.euler_vv * handles.dm';
        qq(length(C_vv)/2 , 2) = 0;
        for idx = 1:2:length(C_vv)
            qq(idx, 1)     = C_vv(idx, idx);
            qq(idx, 2)     = C_vv(idx, idx + 1);
            
            qq(idx + 1, 1) = C_vv(idx + 1, idx);
            qq(idx + 1, 2) = C_vv(idx + 1, idx + 1);
        end
        
        handles.output_positions  = handles.positions;
        handles.output_velocities = velocities;
        handles.output_residuals  = d_velocities;
        handles.output_sigma      = [sqrt(qq(1:2:end, 1)), sqrt(qq(2:2:end, 2)), qq(1:2:end, 2)];
        
    otherwise
            return;
end

% Write the data in the output table
write_output_tbl(handles.output_tbl, handles.names, handles.output_positions, handles.output_velocities, handles.output_residuals, handles.output_sigma);

% Set the data handle of the output table
handles.output_tbl_data = true;

% Update handles structure
guidata(hObject, handles);



% --------------------------------------------------------------------
function calculate_euler_pole_item_Callback(hObject, eventdata, handles)
% hObject    handle to calculate_euler_pole_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the input data was already loaded
if isequal(handles.input_tbl_data, false)
    errordlg(handles.error_msg_0, 'Error ...',  'Modal');
    return;
end

switch handles.filter_index
    case 1
        % Form the design matrix
        handles.dm = ecef2dm(handles.positions);
        
        % Form the velocity vector
        handles.vv = reshape(handles.velocities', [], 1);
        
        % Form the weight vector
        handles.ww = sigma2ww(handles.velocity_errors, handles.velocity_covar, handles.filter_index);
        if isempty(handles.ww)
            return;
        end
        
        % Calculate the omega vector
        handles.normal = handles.dm' * handles.ww * handles.dm;
        handles.omega = handles.normal \ (handles.dm' * handles.ww * handles.vv);
        
        % Calculate velocities in ECEF
        velocities_ecef = handles.dm * handles.omega;
        velocities_ecef = [velocities_ecef(1:3:end), velocities_ecef(2:3:end), velocities_ecef(3:3:end)];
        handles.d_velocities = handles.velocities - velocities_ecef;
        
        % Calculate velocities in topocentric
        lla = xyz2ell(handles.positions(:, 1), handles.positions(:, 2), handles.positions(:, 3));
        velocities_topo = geocentric2topocentric(velocities_ecef, deg2rad(lla(:, 1:2)));
        d_velocities_topo = geocentric2topocentric(handles.d_velocities, deg2rad(lla(:, 1:2)));
        
        % Calculate the variance-covariance matrix for the omega in the ECEF system
        handles.vv = [handles.vv(1:3:end), handles.vv(2:3:end), handles.vv(3:3:end)];
        dV_ecef = (handles.vv - velocities_ecef)';
        dV_ecef = dV_ecef(:);
        
        handles.df_value = 2 * length(dV_ecef) / 3 - 3;
        
        handles.s0_2 = (dV_ecef' * handles.ww * dV_ecef) / handles.df_value;
        C_ww = handles.s0_2 * (handles.normal\eye(length(handles.normal)));
        
        % Calculate the variance-covariance matrix for velocities
        C_vv = handles.dm * C_ww * handles.dm';
        qq(length(C_vv) , 3) = 0;
        for idx = 1:3:length(C_vv)
            qq(idx, 1)     = C_vv(idx, idx);
            qq(idx, 2)     = C_vv(idx, idx + 1);
            qq(idx, 3)     = C_vv(idx, idx + 2);
            
            qq(idx + 1, 1) = C_vv(idx + 1, idx);
            qq(idx + 1, 2) = C_vv(idx + 1, idx + 1);
            qq(idx + 1, 3) = C_vv(idx + 1, idx + 2);
            
            qq(idx + 2, 1) = C_vv(idx + 2, idx);
            qq(idx + 2, 2) = C_vv(idx + 2, idx + 1);
            qq(idx + 2, 3) = C_vv(idx + 2, idx + 2);            
        end
        
        % Convert velocity variances to the topocentric system
        s_velocities = geocentric2topocentric_covmat(qq, deg2rad(lla(:, 1:2)));
        
        handles.output_positions  = lla(:, 1:2);
        handles.output_velocities = velocities_topo(:, 1:2);
        handles.output_residuals  = d_velocities_topo(:, 1:2);
        handles.output_sigma      = [sqrt(s_velocities(1:3:end, 1)), sqrt(s_velocities(2:3:end, 2)), s_velocities(1:3:end, 2)];
        
        % Calculate RMSE_TEXT
        handles.rmse_value = sqrt(dV_ecef' * handles.ww * dV_ecef / length(dV_ecef));
        
    case 2
        % Form the design matrix
        handles.dm = handles.earth_radius * topo2dm(deg2rad(handles.positions));
        
        % Form the velocity vector
        handles.vv = reshape(handles.velocities', [], 1);
        
        % Form the weight vector
        handles.ww = sigma2ww(handles.velocity_errors, handles.velocity_covar, handles.filter_index);
        if isempty(handles.ww)
            return;
        end
        
        % Calculate the omega vector
        handles.normal = handles.dm' * handles.ww * handles.dm;
        handles.omega = handles.normal \ (handles.dm' * handles.ww * handles.vv);
        
        % Calculate velocities in topocentric system
        velocities_topo = handles.dm * handles.omega;
        velocities_topo = [velocities_topo(1:2:end), velocities_topo(2:2:end)];
        handles.d_velocities = handles.velocities - velocities_topo;
        
        % Calculate the variance-covariance matrix for the omega in the topocentric system
        handles.vv = [handles.vv(1:2:end), handles.vv(2:2:end)];
        dV_topo = (handles.vv - velocities_topo)';
        dV_topo = dV_topo(:);
        
        handles.df_value = length(dV_topo) - 3;
        
        handles.s0_2 = (dV_topo' * handles.ww * dV_topo) / handles.df_value;
        C_ww = handles.s0_2 * (handles.normal\eye(length(handles.normal)));
        
        % Calculate the variance-covariance matrix for velocities
        C_vv = handles.dm * C_ww * handles.dm';
        qq(length(C_vv) , 2) = 0;
        for idx = 1:2:length(C_vv)
            qq(idx, 1)     = C_vv(idx, idx);
            qq(idx, 2)     = C_vv(idx, idx + 1);
            
            qq(idx + 1, 1) = C_vv(idx + 1, idx);
            qq(idx + 1, 2) = C_vv(idx + 1, idx + 1);
        end
        
        handles.output_positions  = handles.positions;
        handles.output_velocities = velocities_topo;
        handles.output_residuals  = handles.d_velocities;
        handles.output_sigma      = [sqrt(qq(1:2:end, 1)), sqrt(qq(2:2:end, 2)), qq(1:2:end, 2)];

        % Calculate RMSE_TEXT
        handles.rmse_value = sqrt(dV_topo' * handles.ww * dV_topo / length(dV_topo));
        
    otherwise
            return;
end

% Write the data in the output table
write_output_tbl(handles.output_tbl, handles.names, handles.output_positions, handles.output_velocities, handles.output_residuals, handles.output_sigma);

% Set the data handle of the output table
handles.output_tbl_data = true;

% Calculate the Euler parameters
omega  = rad2deg( sqrt(handles.omega' * handles.omega) ) * 10.^6;
lambda = rad2deg( atan2(handles.omega(2), handles.omega(1)) );
phi    = rad2deg( atan2(handles.omega(3), sqrt(handles.omega(1).^2 + handles.omega(2).^2)));

% Put the Euler parameters in the uitable
set(handles.euler_pole_omega, 'String', num2str(omega));
set(handles.euler_pole_longitude, 'String', num2str(lambda));
set(handles.euler_pole_latitude, 'String', num2str(phi));

% Calculate accuracy of the Euler parameters
omega_length = sqrt(handles.omega' * handles.omega);
H = [handles.omega(1)/omega_length, handles.omega(2)/omega_length, handles.omega(3)/omega_length;
    -handles.omega(2)/(handles.omega(1)^2 + handles.omega(2)^2), handles.omega(1)/(handles.omega(1)^2 + handles.omega(2)^2), 0;
    -handles.omega(1)*handles.omega(3) / (omega_length^2 * sqrt(handles.omega(1)^2 + handles.omega(2)^2)), -handles.omega(2) * handles.omega(3) / (omega_length^2 * sqrt(handles.omega(1)^2 + handles.omega(2)^2)), sqrt(handles.omega(1)^2 + handles.omega(2)^2) / omega_length^2];
C_omega = H * C_ww * H';
sigma_omega = sqrt(diag(C_omega));

% Put accuracy of the Euler parameters in the uitable
set(handles.euler_pole_omega_sigma, 'String', num2str(rad2deg(sigma_omega(1))*10^6));
set(handles.euler_pole_longitude_sigma, 'String', num2str(rad2deg(sigma_omega(2))));
set(handles.euler_pole_latitude_sigma, 'String', num2str(rad2deg(sigma_omega(3))));

% Write statistics
set(handles.df, 'String', num2str(handles.df_value));
set(handles.apost_sigma, 'String', num2str(sqrt(handles.s0_2)));
set(handles.rmse, 'String', num2str(handles.rmse_value));

% Check if the a-posteriori sigma value is negative
if handles.s0_2 < 0
    warndlg(handles.error_msg_3, 'Warning ...',  'Modal');
end    

% Update handles structure
guidata(hObject, handles);



function euler_pole_latitude_Callback(hObject, eventdata, handles)
% hObject    handle to euler_pole_latitude (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of euler_pole_latitude as text
%        str2double(get(hObject,'String')) returns contents of euler_pole_latitude as a double


% --- Executes during object creation, after setting all properties.
function euler_pole_latitude_CreateFcn(hObject, eventdata, handles)
% hObject    handle to euler_pole_latitude (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function euler_pole_longitude_Callback(hObject, eventdata, handles)
% hObject    handle to euler_pole_longitude (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of euler_pole_longitude as text
%        str2double(get(hObject,'String')) returns contents of euler_pole_longitude as a double


% --- Executes during object creation, after setting all properties.
function euler_pole_longitude_CreateFcn(hObject, eventdata, handles)
% hObject    handle to euler_pole_longitude (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function euler_pole_omega_Callback(hObject, eventdata, handles)
% hObject    handle to euler_pole_omega (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of euler_pole_omega as text
%        str2double(get(hObject,'String')) returns contents of euler_pole_omega as a double


% --- Executes during object creation, after setting all properties.
function euler_pole_omega_CreateFcn(hObject, eventdata, handles)
% hObject    handle to euler_pole_omega (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function euler_pole_latitude_sigma_Callback(hObject, eventdata, handles)
% hObject    handle to euler_pole_latitude_sigma (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of euler_pole_latitude_sigma as text
%        str2double(get(hObject,'String')) returns contents of euler_pole_latitude_sigma as a double


% --- Executes during object creation, after setting all properties.
function euler_pole_latitude_sigma_CreateFcn(hObject, eventdata, handles)
% hObject    handle to euler_pole_latitude_sigma (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function euler_pole_longitude_sigma_Callback(hObject, eventdata, handles)
% hObject    handle to euler_pole_longitude_sigma (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of euler_pole_longitude_sigma as text
%        str2double(get(hObject,'String')) returns contents of euler_pole_longitude_sigma as a double


% --- Executes during object creation, after setting all properties.
function euler_pole_longitude_sigma_CreateFcn(hObject, eventdata, handles)
% hObject    handle to euler_pole_longitude_sigma (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function euler_pole_omega_sigma_Callback(hObject, eventdata, handles)
% hObject    handle to euler_pole_omega_sigma (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of euler_pole_omega_sigma as text
%        str2double(get(hObject,'String')) returns contents of euler_pole_omega_sigma as a double


% --- Executes during object creation, after setting all properties.
function euler_pole_omega_sigma_CreateFcn(hObject, eventdata, handles)
% hObject    handle to euler_pole_omega_sigma (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --------------------------------------------------------------------
function view_velocity_vectors_item_Callback(hObject, eventdata, handles)
% hObject    handle to view_velocity_vectors_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the output table is already filled
if isequal(handles.output_tbl_data, false)
    errordlg(handles.error_msg_1, 'Error ...',  'Modal');
    return;
elseif strcmpi(get(handles.view_velocity_vectors_item, 'Checked'), 'On')
    set(handles.view_velocity_vectors_item, 'Checked', 'off');
    delete(handles.quiver);
    return;
end

% Plot the velocity vectors
hold(handles.map_canvas, 'on');
handles.quiver = quiver(handles.map_canvas, handles.output_positions(:, 2), handles.output_positions(:, 1), handles.output_velocities(:, 2), handles.output_velocities(:, 1), 'r');
hold(handles.map_canvas, 'off');

% Set the menu item checked
set(handles.view_velocity_vectors_item, 'Checked', 'On');

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function view_stations_names_item_Callback(hObject, eventdata, handles)
% hObject    handle to view_stations_names_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the input table is already loaded
if isequal(handles.input_tbl_data, false)
    errordlg(handles.error_msg_0, 'Error ...',  'Modal');
    return;
elseif strcmpi(get(handles.view_stations_names_item, 'Checked'), 'On')
    set(handles.view_stations_names_item, 'Checked', 'off');
    delete(handles.stn_names);
    return;
end

% Get the stations coordinates
switch handles.filter_index
    case 1
        stn_coordinates = xyz2ell(handles.positions(:, 1), handles.positions(:, 2), handles.positions(:, 3));
        
    case 2
        stn_coordinates = handles.positions;
        
    otherwise
            return;
            
end

% Plot the names
hold(handles.map_canvas, 'on');
handles.stn_names = text(stn_coordinates(:, 2), stn_coordinates(:, 1), ...
    handles.names, ...
    'Parent', handles.map_canvas, ...
    'Clipping', 'On', ...
    'Color', [0.2, 0.2, 0.6], ...
    'FontAngle', 'Italic', ...
    'FontSize', 8);
hold(handles.map_canvas, 'off');

% Set the menu item checked
set(handles.view_stations_names_item, 'Checked', 'On');

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function view_euler_pole_item_Callback(hObject, eventdata, handles)
% hObject    handle to view_euler_pole_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get the Euler parameters
handles.euler_latitude = str2double(get(handles.euler_pole_latitude, 'String'));
handles.euler_longitude = str2double(get(handles.euler_pole_longitude, 'String'));

% Check if any of the Euler parameters is empty
if any(isnan([handles.euler_latitude, handles.euler_longitude]))
    errordlg(handles.error_msg_2, 'Error ...',  'Modal');
    return;
elseif any([~isbetween(handles.euler_latitude, -90, 90); ~isbetween(handles.euler_longitude, -180, 180)])
    errordlg(handles.error_msg_5, 'Error ...',  'Modal');
    return;
elseif strcmpi(get(handles.view_euler_pole_item, 'Checked'), 'On')
    set(handles.view_euler_pole_item, 'Checked', 'Off');
    delete(handles.euler_pole);
    return;   
end

% Plot the Euler pole
hold(handles.map_canvas, 'on');
handles.euler_pole = plot(handles.map_canvas, handles.euler_longitude, handles.euler_latitude, 'o',...
        'MarkerEdgeColor','k',...
        'MarkerFaceColor','r');
hold(handles.map_canvas, 'off');

% Set the menu item checked
set(handles.view_euler_pole_item, 'Checked', 'On');

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function save_item_Callback(hObject, eventdata, handles)
% hObject    handle to save_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function save_output_table_item_Callback(hObject, eventdata, handles)
% hObject    handle to save_output_table_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the output table is already filled
if isequal(handles.output_tbl_data, false)
    errordlg(handles.error_msg_1, 'Error ...',  'Modal');
    return;
end

% Form the numerical part of the output data
output_tbl_data = [handles.output_positions, handles.output_velocities, handles.output_residuals, handles.output_sigma];

% Create and open a file to write the output data
[file_name, path_name, filter_index] = uiputfile(...
    {'*.csv', 'Comma-Separated Values File (*.csv)'; ...
    '*.shp', 'ESRI Shape File (*.shp)'; ...
    '*.kml', 'Keyhole Markup Language File (*.kml)'}, 'Save output data ...');
if filter_index
    cd(path_name);
end

switch filter_index
    case 1
        f_id = fopen(fullfile(path_name, file_name), 'wt');
        fprintf(f_id, '# Velocity field data\n');
        fprintf(f_id, '# Format: Station, Lat(deg), Long(deg), Vn(m/yr), Ve(m/yr), dVn(m/yr), dVe(m/yr), Sigma Vn, Sigma Ve, Sigma Vne\n');
        for idx = 1:length(output_tbl_data)
            fprintf(f_id, '%s, %11.6f, %11.6f, %11.9f, %11.9f, %11.9f, %11.9f, %11.9f, %11.9f, %11.9f\n', ...
                handles.names{idx}, output_tbl_data(idx, :));
        end
        fclose(f_id);

    case 2
        output_tbl_geo_struct = vel_field2geo_struct(handles.names, output_tbl_data);
        shapewrite(output_tbl_geo_struct, fullfile(path_name, file_name));
        
    case 3
        output_tbl_geo_struct = vel_field2geo_struct(handles.names, output_tbl_data);
        kmlwrite(fullfile(path_name, file_name), output_tbl_geo_struct);

    otherwise
            return;
end

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function save_euler_pole_parameters_item_Callback(hObject, eventdata, handles)
% hObject    handle to save_euler_pole_parameters_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get the Euler parameters
phi = str2double(get(handles.euler_pole_latitude, 'String'));
lambda = str2double(get(handles.euler_pole_longitude, 'String'));
omega = str2double(get(handles.euler_pole_omega, 'String'));

phi_sigma = str2double(get(handles.euler_pole_latitude_sigma, 'String'));
lambda_sigma = str2double(get(handles.euler_pole_longitude_sigma, 'String'));
omega_sigma = str2double(get(handles.euler_pole_omega_sigma, 'String'));

% Check if any of the Euler parameters is empty
if any(isnan([phi, lambda, omega, phi_sigma, lambda_sigma, omega_sigma]))
    errordlg(handles.error_msg_2, 'Error ...',  'Modal');
    return;
elseif any([~isbetween(phi, -90, 90); ~isbetween(lambda, -180, 180); ~gt([omega; phi_sigma; lambda_sigma; omega_sigma], 0)])
    errordlg(handles.error_msg_5, 'Error ...',  'Modal');
    return;
end

% Create and open a file to write the parameters
[file_name, path_name, filter_index] = uiputfile(...
    {'*.csv', 'Comma-Separated Values File (*.csv)'; ...
    '*.kml', 'Keyhole Markup Language File (*.kml)'}, 'Save Euler pole parameters ...');

if filter_index
    cd(path_name);
end

switch filter_index
    case 1
        f_id = fopen(fullfile(path_name, file_name), 'wt');
        fprintf(f_id, '# Euler pole parameters\n');
        fprintf(f_id, '# Lat(deg), Long(deg), Omega(deg/Myr), Sigma Lat, Sigma Long, Sigma Omega\n');
        fprintf(f_id, '%0f, %0f, %0f, %0f, %0f, %0f', phi, lambda, omega, phi_sigma, lambda_sigma, omega_sigma);
        fclose(f_id);

    case 2
        geo_struct.Geometry = 'Point';
        geo_struct.Name = ['Euler Pole for ', file_name];
        geo_struct.Lat = phi;
        geo_struct.Lon = lambda;
        geo_struct.Latitude = geo_struct.Lat;
        geo_struct.Longitude = geo_struct.Lon;
        geo_struct.Velocity  = omega;

        kmlwrite(fullfile(path_name, file_name), geo_struct);
        
    otherwise
            return;
end

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function view_res_velocity_vectors_item_Callback(hObject, eventdata, handles)
% hObject    handle to view_res_velocity_vectors_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the output table is already filled
if isequal(handles.output_tbl_data, false)
    errordlg(handles.error_msg_1, 'Error ...',  'Modal');
    return;
elseif strcmpi(get(handles.view_res_velocity_vectors_item, 'Checked'), 'On')
    set(handles.view_res_velocity_vectors_item, 'Checked', 'off');
    delete(handles.quiver_diff);
    return;
end

hold(handles.map_canvas, 'on');
handles.quiver_diff = quiver(handles.map_canvas, handles.output_positions(:, 2), handles.output_positions(:, 1), handles.output_residuals(:, 2), handles.output_residuals(:, 1), 'b');
hold(handles.map_canvas, 'off');

% Set the menu item checked
set(handles.view_res_velocity_vectors_item, 'Checked', 'On');

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function reload_pushtool_ClickedCallback(hObject, eventdata, handles)
% hObject    handle to reload_pushtool (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

handles = write_input_tbl(hObject, eventdata, handles);

% Update handles structure
guidata(hObject, handles);


% --- Executes when user attempts to close epc.
function epc_CloseRequestFcn(hObject, eventdata, handles)
% hObject    handle to epc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

exit_item_Callback(hObject, eventdata, handles);

return;


% --------------------------------------------------------------------
function save_pushtool_ClickedCallback(hObject, eventdata, handles)
% hObject    handle to save_pushtool (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

save_output_table_item_Callback(hObject, eventdata, handles);

return;

% --------------------------------------------------------------------
function view_world_map_item_Callback(hObject, eventdata, handles)
% hObject    handle to view_world_map_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% Get the output data index

if strcmpi(get(handles.view_world_map_item, 'Checked'), 'On')
    set(handles.view_world_map_item, 'Checked', 'Off');
    set(handles.world_map, 'Visible', 'Off');
else
    set(handles.view_world_map_item, 'Checked', 'On');
    set(handles.world_map, 'Visible', 'On');
end

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function chi_square_test_item_Callback(hObject, eventdata, handles)
% hObject    handle to chi_square_test_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the output table is already filled
if isequal(handles.output_tbl_data, false)
    errordlg(handles.error_msg_1, 'Error ...',  'Modal');
    return;
end

% Get the velocity residuals
dV_n = handles.output_residuals(:, 1);
dV_e = handles.output_residuals(:, 2);

% Chi-square goodness-of-fit test
[h_n, p_n, stats_n] = chi2gof(dV_n(:));
[h_e, p_e, stats_e] = chi2gof(dV_e(:));

% Report the test results
if isequal(h_n, 0)
    test_result_n = 'cannot be rejected.';
else
    test_result_n = 'can be rejected.';
end

if isequal(h_e, 0)
    test_result_e = 'cannot be rejected.';
else
    test_result_e = 'can be rejected.';
end

output_str_1 = 'North Velocity Residuals:';
output_str_2 = sprintf('\n   The null hypothesis %s\n   P value: %0.10f\n   Chi-square statistic: %f\n   Degrees of freedom: %d\n\n', ...
    test_result_n, p_n, stats_n.chi2stat, stats_n.df);
output_str_3 = 'East Velocity Residuals:';
output_str_4 = sprintf('\n   The null hypothesis %s\n   P value: %0.10f\n   Chi-square statistic: %f\n   Degrees of freedom: %d\n', ...
    test_result_e, p_e, stats_e.chi2stat, stats_e.df);
output_str = [output_str_1, output_str_2, output_str_3, output_str_4];

msgbox(output_str, 'Chi-square test results ...', 'Modal');

% Update handles structure
guidata(hObject, handles);



function rmse_Callback(hObject, eventdata, handles)
% hObject    handle to rmse_text (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of rmse as text
%        str2double(get(hObject,'String')) returns contents of rmse as a double


% --- Executes during object creation, after setting all properties.
function rmse_CreateFcn(hObject, eventdata, handles)
% hObject    handle to rmse_text (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function apost_sigma_Callback(hObject, eventdata, handles)
% hObject    handle to apost_sigma (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of apost_sigma as text
%        str2double(get(hObject,'String')) returns contents of apost_sigma as a double


% --- Executes during object creation, after setting all properties.
function apost_sigma_CreateFcn(hObject, eventdata, handles)
% hObject    handle to apost_sigma (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function df_Callback(hObject, eventdata, handles)
% hObject    handle to df (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of df as text
%        str2double(get(hObject,'String')) returns contents of df as a double


% --- Executes during object creation, after setting all properties.
function df_CreateFcn(hObject, eventdata, handles)
% hObject    handle to df (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function nb_obs_Callback(hObject, eventdata, handles)
% hObject    handle to nb_stns (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of nb_stns as text
%        str2double(get(hObject,'String')) returns contents of nb_stns as a double


% --- Executes during object creation, after setting all properties.
function nb_stns_CreateFcn(hObject, eventdata, handles)
% hObject    handle to nb_stns (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --------------------------------------------------------------------
function corr_coef_test_item_Callback(hObject, eventdata, handles)
% hObject    handle to corr_coef_test_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the output table is already filled
if isequal(handles.output_tbl_data, false)
    errordlg(handles.error_msg_1, 'Error ...',  'Modal');
    return;
end

% Get the velocities
switch handles.filter_index
    case 1
        v_obs = geocentric2topocentric(handles.velocities, deg2rad(handles.output_positions));
        v_obs = v_obs(:, 1:2);
    case 2
        v_obs = handles.velocities;
    otherwise
        return;
end

v_model = handles.output_velocities;

% Clear the canvas
cla(handles.test_canvas);

% Plot the points
handles.corr_coef_plot = plot(handles.test_canvas, ...
    v_obs(:, 1), v_model(:, 1), 'b.', ...
    v_obs(:, 2), v_model(:, 2), 'r.');

set(handles.corr_coef_plot, ...
    {'DisplayName'}, {'V_N';'V_E'}, ...
    'UIContextMenu', handles.data_tip);

labels = cellstr(handles.names);
hold(handles.test_canvas, 'on');
for idx = 1: length( v_obs(:, 1))
    h1 = plot(handles.test_canvas, v_obs(idx, 1), v_model(idx, 1), 'b.', 'Tag', char(labels(idx)), 'UIContextMenu', handles.data_tip);
    h2 = plot(handles.test_canvas, v_obs(idx, 2), v_model(idx, 2), 'r.', 'Tag', char(labels(idx)), 'UIContextMenu', handles.data_tip);
    
    hLegendEntry1 = get(get(h1, 'Annotation'), 'LegendInformation');
    hLegendEntry2 = get(get(h2, 'Annotation'), 'LegendInformation');
    
    set(hLegendEntry1, 'IconDisplayStyle', 'off');
    set(hLegendEntry2, 'IconDisplayStyle', 'off');
end
hold(handles.test_canvas, 'off');

% Set the canvas properties
set(handles.test_canvas, ...
    'YAxisLocation', 'Right', ...
    'Clipping', 'On');

% Write axes labels
xlabel(handles.test_canvas, 'V_{observed}', ...
    'Clipping', 'On', ...
    'Units', 'normalized',...
    'Position', [0.5 0.1]);
ylabel(handles.test_canvas, 'V_{modelled}', ...
    'Clipping', 'On', ...
    'Units', 'normalized',...
    'Position', [0.94 0.5]);

% Plot the legend
h_leg=legend(handles.test_canvas, 'Show');
set(h_leg, ...
    'Orientation', 'Horizontal', ...
    'Location', 'Best', ...
    'Clipping', 'On', ...
    'Box', 'Off')

% Calculate correlations
corr_coef_n = corrcoef(v_obs(:, 1), v_model(:, 1));
corr_coef_e = corrcoef(v_obs(:, 2), v_model(:, 2));

% Perform the t-test
t_calc_n = corr_coef_n(1, 2) * sqrt((length(v_obs(:, 1)) - 2) / (1 - corr_coef_n(1, 2).^2));
t_calc_e = corr_coef_e(1, 2) * sqrt((length(v_obs(:, 2)) - 2) / (1 - corr_coef_e(1, 2).^2));

t_crit_n = abs(tinv(handles.t_test_sig_level, length(v_obs(:, 1)) - 2));
t_crit_e = abs(tinv(handles.t_test_sig_level, length(v_obs(:, 2)) - 2));

% Report the test results
output_str_1 = sprintf('Pearson''s correlation coefficient for:\n');
output_str_2 = sprintf('   North velocity residuals: %f\n   Calculated t statistic: %f\n   Critical t statistic: %f\n\n', corr_coef_n(1, 2), t_calc_n, t_crit_n);
output_str_3 = sprintf('   East velocity residuals: %f\n   Calculated t statistic: %f\n   Critical t statistic: %f\n', corr_coef_e(1, 2), t_calc_e, t_crit_e);
output_str = [output_str_1, output_str_2, output_str_3];

msgbox(output_str, 'Pearson''s correlation coefficient ...', 'Modal');

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function view_tectonic_map_item_Callback(hObject, eventdata, handles)
% hObject    handle to view_tectonic_map_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if strcmpi(get(handles.view_tectonic_map_item, 'Checked'), 'On')
    set(handles.view_tectonic_map_item, 'Checked', 'Off');
    set(handles.tectonic_map, 'Visible', 'Off');
else
    set(handles.view_tectonic_map_item, 'Checked', 'On');
    set(handles.tectonic_map, 'Visible', 'On');
end

% Update handles structure
guidata(hObject, handles);


% --- Executes when epc is resized.
function epc_ResizeFcn(hObject, eventdata, handles)
% hObject    handle to epc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% set(handles.input_tbl, 'Units', 'Normalized');
% set(handles.output_tbl, 'Units', 'Normalized');
% 
% set(handles.output_tbl_title, 'Units', 'Normalized');
% set(handles.input_tbl_title, 'Units', 'Normalized');
% 
% set(handles.map_canvas, 'Units', 'Normalized');
% set(handles.test_canvas, 'Units', 'Normalized');
% 
% set(findobj(handles.status_bar, '-depth', 2), 'Units', 'Normalized');
% 
% set(findobj(handles.euler_pole_uipanel), 'Units', 'Normalized');
% set(findobj(handles.statistics_uipanel), 'Units', 'Normalized');

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function data_snooping_item_Callback(hObject, eventdata, handles)
% hObject    handle to data_snooping_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the output table is already filled
if isequal(handles.output_tbl_data, false)
    errordlg(handles.error_msg_1, 'Error ...',  'Modal');
    return;
end

% Check if the a-posteriori sigma value exists and is positive
if ~isfield(handles, 's0_2')
    errordlg(handles.error_msg_6, 'Error ...',  'Modal');
    return;
elseif handles.s0_2 < 0
    warndlg(handles.error_msg_3, 'Warning ...',  'Modal');
end    

s0 = sqrt(handles.s0_2);
v_res = reshape(handles.d_velocities', [], 1);
q_vv = handles.ww \ eye(size(handles.ww)) - handles.dm * (handles.normal \ eye(size(handles.normal))) * handles.dm';

t_calc = v_res ./ (s0 * sqrt(diag(q_vv)));
t_calc = reshape(t_calc, size(handles.d_velocities, 2), [])';
t_crit = abs(tinv(handles.t_test_sig_level / 2, handles.df_value));

accepted_pnts = isbetween(t_calc, -t_crit, t_crit);
[rejected_pnts_row_idx, rejected_pnts_col_idx] = find(accepted_pnts == 0);
rejected_pnts_row_idx = unique(rejected_pnts_row_idx);

if ~isempty(rejected_pnts_row_idx)
    rejected_pnts = [t_calc(rejected_pnts_row_idx, :), sum(t_calc(rejected_pnts_row_idx, :).*t_calc(rejected_pnts_row_idx, :), 2)];
    [rejected_pnts, sort_idx] = sortrows(rejected_pnts, -size(rejected_pnts, 2));

    names = handles.names(rejected_pnts_row_idx);
    names = names(sort_idx, :);

    if handles.filter_index == 1
        column_names = {'Stations', '<html>sr<sub>X</sub></html>', '<html>sr<sub>Y</sub></html>', '<html>sr<sub>Z</sub></html>'};
    else
        column_names = {'Stations', '<html>sr<sub>N</sub></html>', '<html>sr<sub>E</sub></html>'};
    end

    output_data = [names, num2cell(rejected_pnts(:, 1:end-1))];
    data_snooping_test_dlg(column_names, output_data);
    
else
    msgbox('No point meets the rejection criteria.', 'Data snooping test ...', 'Modal');
end

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function data_tip_Callback(hObject, eventdata, handles)
% hObject    handle to data_tip (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

pnt_label = get(gco, 'Tag');
set(handles.pnt_label, 'label', pnt_label);

% Update handles structure
guidata(hObject, handles);

% --------------------------------------------------------------------
function pnt_label_Callback(hObject, eventdata, handles)
% hObject    handle to pnt_label (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

res_show = get(handles.view_res_velocity_vectors_item, 'Checked');

if strcmpi(res_show, 'on')
    idx = find(strcmpi(handles.names, get(gco, 'Tag')));
    lat = handles.output_positions(idx, 1);
    long = handles.output_positions(idx, 2);
    
    aspect_ratio = diff(get(handles.map_canvas, 'xlim')) / ...
        diff(get(handles.map_canvas, 'ylim'));

    set(handles.map_canvas, ...
        'xlim', long + aspect_ratio .* [-1, 1], ...
        'ylim', lat + [-1, 1]);
end

% Update handles structure
guidata(hObject, handles);


% --------------------------------------------------------------------
function calculate_euler_pole_btn_ClickedCallback(hObject, eventdata, handles)
% hObject    handle to calculate_euler_pole_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

calculate_euler_pole_item_Callback(hObject, eventdata, handles);

return;

% --------------------------------------------------------------------
function calculate_velocities_btn_ClickedCallback(hObject, eventdata, handles)
% hObject    handle to calculate_velocities_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

calculate_velocities_item_Callback(hObject, eventdata, handles);

return;


% --------------------------------------------------------------------
function tau_test_item_Callback(hObject, eventdata, handles)
% hObject    handle to tau_test_item (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if the output table is already filled
if isequal(handles.output_tbl_data, false)
    errordlg(handles.error_msg_1, 'Error ...',  'Modal');
    return;
end

% Check if the a-posteriori sigma value exists and is positive
if ~isfield(handles, 's0_2')
    errordlg(handles.error_msg_6, 'Error ...',  'Modal');
    return;
elseif handles.s0_2 < 0
    warndlg(handles.error_msg_3, 'Warning ...',  'Modal');
end    

s0 = sqrt(handles.s0_2);
v_res = reshape(handles.d_velocities', [], 1);
q_vv = handles.ww \ eye(size(handles.ww)) - handles.dm * (handles.normal \ eye(size(handles.normal))) * handles.dm';

tau_calc = v_res ./ (s0 * sqrt(diag(q_vv)));
tau_calc = reshape(tau_calc, size(handles.d_velocities, 2), [])';
t_crit = abs(tinv(handles.t_test_sig_level / 2, handles.df_value - 1));
tau_crit = t_crit * sqrt(handles.df_value) /sqrt(handles.df_value - 1 + t_crit.^2);

accepted_pnts = isbetween(tau_calc, -tau_crit, tau_crit);
[rejected_pnts_row_idx, rejected_pnts_col_idx] = find(accepted_pnts == 0);
rejected_pnts_row_idx = unique(rejected_pnts_row_idx);

if ~isempty(rejected_pnts_row_idx)
    rejected_pnts = [tau_calc(rejected_pnts_row_idx, :), sum(tau_calc(rejected_pnts_row_idx, :).*tau_calc(rejected_pnts_row_idx, :), 2)];
    [rejected_pnts, sort_idx] = sortrows(rejected_pnts, -size(rejected_pnts, 2));

    names = handles.names(rejected_pnts_row_idx);
    names = names(sort_idx, :);

    if handles.filter_index == 1
        column_names = {'Stations', '<html>sr<sub>X</sub></html>', '<html>sr<sub>Y</sub></html>', '<html>sr<sub>Z</sub></html>'};
    else
        column_names = {'Stations', '<html>sr<sub>N</sub></html>', '<html>sr<sub>E</sub></html>'};
    end

    output_data = [names, num2cell(rejected_pnts(:, 1:end-1))];
    tau_test_dlg(column_names, output_data);
    
else
    msgbox('No point meets the rejection criteria.', 'Tau test ...', 'Modal');
end

% Update handles structure
guidata(hObject, handles);
