function [names, positions, velocitys, velocity_errors, velocity_covar] = read_neu_file(f_id)
    f = textscan(f_id, '%s%s%s%s%s%s%s%s', 'CommentStyle', '#');
    
    names = f{1};
    positions = [str2double(f{2}), str2double(f{3})];
    velocitys = [str2double(f{4}), str2double(f{5})];
    velocity_errors = [str2double(f{6}), str2double(f{7})];
    velocity_covar = str2double(f{8});
    
    % Check position and velocity vectors for NaNs
    nan_idx = isnan([positions, velocitys]);
    if any(nan_idx(:))
        [row_idx, col_idx] = find(nan_idx);
        row_idx = unique(row_idx);
        
        names_nan_list = char(names(row_idx(1)));
        for idx = 2:length(row_idx)
            names_nan_list = sprintf('%s, %s', names_nan_list, char(names(row_idx(idx))));
        end
        output_str = sprintf('Because of NaN values in their records, the following stations were ignored:\n%s.', ...
            names_nan_list);
        warndlg(output_str, 'Warning ...', 'Modal');
        
        names(row_idx) = [];
        positions(row_idx, :) = [];
        velocitys(row_idx, :) = [];
        velocity_errors(row_idx, :) = [];
        velocity_covar(row_idx, :) = [];
    end
    
return;