function ww = sigma2ww(s_vect, covar_vect, file_type_index)

ww = [];

switch file_type_index
    case 1
        s_col_idx = 1:3;
        covar_col_idx = 1:3;
        
    case 2
        s_col_idx = 1:2;
        covar_col_idx = 1;
        
    otherwise
            return;
end

if and(any(any(~isnan(s_vect(:, s_col_idx)))), any(any(isnan(s_vect(:, s_col_idx))))) || and(any(any(~isnan(covar_vect(:, covar_col_idx)))), any(any(isnan(covar_vect(:, covar_col_idx)))))
    errordlg('Combination of real and NaN values for sigma/covariance in the input table is not allowed.', 'Error ...', 'Modal');
    return;
end


% Substitute NaNs in sigmas with 1
s_vect(isnan(s_vect(:, s_col_idx))) = 1;

% Substitute NaNs in covariances with 0
covar_vect(isnan(covar_vect(:, covar_col_idx))) = 0;

% Make s zero covariance matrix
[n_row, n_col] = size(s_vect);
cc = zeros(n_row * n_col);

% Reshape matrices into vectors
s_vect      = reshape(s_vect(:, s_col_idx)', [], 1);
covar_vect  = reshape(covar_vect(:, covar_col_idx)', [], 1);

% Form the covariance matrix
switch file_type_index
    case 1
        for idx = 1:n_col:n_row * n_col - 2
            cc(idx, idx)         = s_vect(idx).^2;
            cc(idx + 1, idx + 1) = s_vect(idx + 1).^2;
            cc(idx + 2, idx + 2) = s_vect(idx + 2).^2;

            cc(idx, idx + 1) = covar_vect(idx);
            cc(idx + 1, idx) = covar_vect(idx);

            cc(idx, idx + 2) = covar_vect(idx + 1);
            cc(idx + 2, idx) = covar_vect(idx + 1);

            cc(idx + 1, idx + 2) = covar_vect(idx + 2);
            cc(idx + 2, idx + 1) = covar_vect(idx + 2);

        end    

    case 2
        for idx = 1:n_col:n_row * n_col - 1
            cc(idx, idx)         = s_vect(idx).^2;
            cc(idx + 1, idx + 1) = s_vect(idx + 1).^2;

            cc(idx, idx + 1) = covar_vect((idx + 1) / 2);
            cc(idx + 1, idx) = covar_vect((idx + 1) / 2);

        end    
end

if issingular(cc)
    errordlg('The covariance matrix is singular, close to singular or badly scaled. So, it is replaced by an identity matrix. Results may be inaccurate.', 'Error ...', 'Modal');
    ww = eye(size(cc));
else
    ww = cc \ eye(size(cc));
end

return;